/************************************************************************************
DeviceControl RT-Component
Copyright (c) 2010, Kyoto Uni.
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD lisence.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

/***********************************************************************************/
/*!
*	@file myTimer.h
*	@brief Class for timer
*	@author kazuyuki kon@ Kyoto-u
*/
/***********************************************************************************/
#ifdef WIN32
	#include <Windows.h>
#else
	#include <sys/time.h>
#endif
#include <stdio.h>
#include <iostream>


/***********************************************************************************/
/*!
*	@class myTimer
*/
/***********************************************************************************/
class myTimer
{
private:
	bool isStart;		//!< whether timer is started or not.
	double timeout;		//!< timeout[sec]
	double datatime;	//!< current time[sec]

public:

	/*!
	*	@brief Constructor
	*/
	myTimer()
	{
		Clear();
	}

	/*!
	*	@brief Copy constructor
	*/
	myTimer(double t0)
	{
		Clear();
		timeout = t0;
	}

	/*!
	*	@brief Destructor
	*/
	~myTimer(){}

	/*!
	*	@brief Set current time to this timer
	*/
	void SetCurrentTime(void)
	{
		datatime = GetTime();
		if(!isStart)isStart = true;
	}
	/*!
	*	@brief get current time
	*	@retval current time (from PC )[sec]
	*/
	double GetTime(void)
	{
#ifdef WIN32
	LARGE_INTEGER freq, new_time;
	QueryPerformanceFrequency(&freq);
	QueryPerformanceCounter(&new_time);

	return (double)((new_time.QuadPart)*1000000/freq.QuadPart)/1000000;
#else
		struct timeval t1;
		gettimeofday(&t1,NULL);
		return ToSec(t1);
#endif
	}
	
	/*!
	*	@brief Get time from SetCurrentTime() is called 
	*/
	double GetTimeFromPre(void)
	{
		return GetTime() - datatime;
	}

	/*!
	*	@brief Convert timeval to double[sec]
	*	@param[in] t0 timeval typed one 
	*	@retval time[sec]
	*/
	double ToSec(timeval t0)
	{
		return (double)t0.tv_sec + (double)t0.tv_usec/1000000;
	}

	/*!
	*	@brief Clear timer 
	*/
	void Clear(void)
	{
		isStart = false;
		timeout = 0;
		datatime = 0;
	}

	/*!
	*	@brief Get connection status
	*	@retval 
	*/
	bool isConnect(void)
	{
		if(isStart && GetTimeFromPre() <= timeout)return true;
		else return false;		
	}

	/*!
	*	@brief Set timeout of timer
	*	@param[in] t0 timeout[sec]
	*/
	void SetTimeout(double t0)
	{
		timeout = t0;
	}

};

